/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is XulApp IDE.
 *
 * The Initial Developer of the Original Code is Synesis Software (USA) Inc.
 *

 * Portions created by the Initial Developer are Copyright (C) 2001-2005
 * Synesis Software (USA) Inc. the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * 	Peter Wilson, orignal author.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the LGPL or the GPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 *
 * ***** END LICENSE BLOCK ***** */
netscape.security.PrivilegeManager.enablePrivilege("UniversalXPConnect");
XaLib.include("chrome://xalib/content/XPCOM/XPCOM.js");
XaLib.include("chrome://xalib/content/IO/File/File.js");

function FilePicker
(
	parentWindow, 	// window: The dialog owner window.
	title,			// string: The dialog title
	mode			// enum: FilePicker:OPEN,SAVE,OPEN_FOLDER,OPEN_MULTIPLE.
)

{
	this._mode = mode || FilePicker.OPEN;						
	this._filePicker = XPCOM.getComponent("@mozilla.org/filepicker;1", "nsIFilePicker");
	this._filePicker.init(parentWindow, title, this._mode);
}

FilePicker.prototype.appendFilter = function
(
	title, // string:name of the filter
	filter // string:extensions to filter -- semicolon and space separated
)
{
	this._filePicker.appendFilter(title, filter);
}
FilePicker.prototype.setFilterMask = function
(
	filterMask // bitmask of predefind filters i.e. (FilePicker.ALL | FilePicker.HTML)
)
{
	this._filePicker.appendFilters(filterMask);
}

FilePicker.prototype.__defineSetter__('filterIndex', function(index) {
	this._filePicker.filterIndex = index;
})
FilePicker.prototype.__defineGetter__('filterIndex', function() {
	return this._filePicker.filterIndex;
})

FilePicker.prototype.__defineSetter__('initialDirectory', function(aFile) {
	if (aFile instanceof Components.interfaces.nsILocalFile)
		this._filePicker.displayDirectory = aFile;
	else
		if (aFile instanceof File)
			this._filePicker.displayDirectory = aFile.localFile;
		else
			if (typeof(aFile) == "string") {
				var newFile = new File(aFile);
				this._filePicker.displayDirectory = newFile.localFile;
			}
			else throw "initialDirectory aFile invalid";
})

FilePicker.prototype.__defineGetter__('initialDirectory', function() {
	return new File(this._filePicker.displayDirectory);
})
FilePicker.prototype.__defineSetter__('initialFilename', function(aString) {
	this._filePicker.defaultString = aString;
})
FilePicker.prototype.__defineSetter__('initialExtension', function(aString) {
	this._filePicker.defaultExtension = aString;
})


FilePicker.prototype.show = function() {
	this._iterator = null;
	this._result = this._filePicker.show();
	
	if (this._result == FilePicker.OK && !this._filePicker.file) {
		this._result = FilePicker.CANCEL;
	}
	else {
		if (this._mode == FilePicker.OPEN_MULTIPLE)
			this._iterator = this._filePicker.files
		
	}

	return this._result;
}

FilePicker.prototype.__defineGetter__('file', function() {
	if (this._result != FilePicker.OK && this._result != FilePicker.OK_REPLACE)
		return null;
		
	if (this._mode == FilePicker.OPEN_MULTIPLE) {
		if (this._iterator) {
			if (this._iterator.hasMoreElements()) {
				var newFile = this._iterator.getNext();
				if (newFile instanceof Components.interfaces.nsILocalFile)
					return new File(newFile.path);
			}
		}
		return null;
	}
	else
		return new File(this._filePicker.file);
})


FilePicker.FP = Components.interfaces.nsIFilePicker;
// Modes:
FilePicker.OPEN 			= FilePicker.FP.modeOpen;			// Choose a file to open.
FilePicker.SAVE 			= FilePicker.FP.modeSave;			// Choose a file (or specify a file) to save.
FilePicker.OPEN_FOLDER 		= FilePicker.FP.modeGetFolder;		// Choose a folder to open.
FilePicker.OPEN_MULTIPLE 	= FilePicker.FP.modeOpenMultiple;	// Choose multiple files to open.
// Return codes:
FilePicker.OK 				= FilePicker.FP.returnOK;			// User hit Ok, process selection
FilePicker.CANCEL 			= FilePicker.FP.returnCancel;		// User hit cancel, ignore selection
FilePicker.OK_REPLACE 		= FilePicker.FP.returnReplace;		// User acknowledged file already exists so ok to replace, process selection
// Predefined filters for use with appendFilters(fillter1|filter2...)
FilePicker.ALL				= FilePicker.FP.filterAll;			// *.*
FilePicker.HTML				= FilePicker.FP.filterHTML;			// *.html; *.htm
FilePicker.TEXT				= FilePicker.FP.filterText;			// *.txt
FilePicker.IMAGES			= FilePicker.FP.filterImages;		// *.png; *.gif; *.jpg; *.jpeg
FilePicker.XML				= FilePicker.FP.filterXML;			// *.xml
FilePicker.XUL				= FilePicker.FP.filterXUL;			// *.xul
FilePicker.APPS				= FilePicker.FP.filterApps;			// Applications (per-platform implementation)

