/*** -*- Mode: Javascript; tab-width: 2;

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is XulApp IDE.
The Initial Developer of the Original Code is Peter Wilson.

Portions created by Peter Wilson are
Copyright (C) 2000 Peter Wilson.  All
Rights Reserved.

************/

XaLib.include("chrome://xalib/content/IO/File/File.js");

File.prototype.createDir = function(permissions) {
	try { 
		if (!permissions) permissions = File.DEFAULT_PERMS;
		this._localFile.create(File.DIRECTORY_TYPE, permissions);
		return this;
	}
	catch (e) { 
		throw "File.create: Error: " + e;
	}
}

File.prototype.clone = function() {
	try { 
		return new File(this._path);
	}
	catch (e) { 
		throw "File.clone: Error: " + e;
	}
}

/**TIP: To get correct native file separator's append one level at a time. **/
File.prototype.append = function(pathToAppend) {
	try {
		return new File(this.path + File.pathSeparator + pathToAppend);
	}
	catch (e) { 
		throw File.getMessage("append.UnknownError", pathToAppend, e, this.path, getStackTrace()); 
	}
}

File.prototype.isExecutable = function() {
	try { 
		if (this._localFile)
			return this._localFile.isExecutable();
		else
			return false;
	}
	catch (e) { 
		throw "File.isSymlink: Error: " + e;
	}
}
File.prototype.createUnique = function(fileType, permissions) {
//TODO
}
File.prototype.contains = function(aFile) {
//TODO
}
File.prototype.copyTo = function(newParentDir, newName, replace) {
	if (!newParentDir && !newName)
		throw File.getMessage("copyTo.NoDestination", getStackTrace() );
		
	if (!newName)
		newName = this.leaf;
	if (!newParentDir)
		newParentDir = this.parent;

	var newFile = newParentDir.append(newName);
	

	if (replace && newFile.exists())
		newFile.remove();
	

	this._localFile.copyTo(newParentDir._localFile, newName);
	return newFile;
}
File.prototype.moveTo = function(newParentDir, newName, replace) {
	if (!newParentDir && !newName)
		throw File.getMessage("moveTo.NoDestination", getStackTrace() );
	

	if (!newName)
		newName = this.leaf;
	if (!newParentDir)
		newParentDir = this.parent;

	var destFile = newParentDir.append(newName);
	

	if (replace) {	
		if (destFile.exists())
			destFile.remove();
	}
	this._localFile.moveTo(newParentDir._localFile, newName);
	// resynchronize other members.
	this.localFile = this._localFile;
	return this;
}
File.prototype.remove = function(recurse) {
	if (!recurse)
		recurse = false;
	this._localFile.remove(recurse);
}

File.prototype.files = function
/* Return an iterator for the directories files.
* The returned iterator can be used as follows:
* <code>
*	var iterator = aDir.files();
*	var aFile;
*   while ((aFile = iterator() )) { // Use double parentheses to suppress strict errors.
*	}
* </code>
*/

(

	filter //RegExp:
) //iterator: returns File objects.

{
	if (this._URI.scheme == "jar")
		return this.entries(filter);
		
	if (!this.isDirectory())
		throw File.getMessage("files.NotADirectory", getStackTrace() );
	if (!this._localFile)
		throw File.getMessage("files.NotALocalFile", getStackTrace() );
		
	var itr = this._localFile.directoryEntries;	
	var iterator = function() {
		while (true) {
			if (!itr.hasMoreElements())
				return null;
			var aFile = itr.getNext();
			if (aFile instanceof Components.interfaces.nsILocalFile) {
				if (!filter)
					break;
				if (filter.test(aFile.leafName))
					break;
			}
		} 
		return new File(aFile);
	}
	return iterator;
}

File.prototype.entries = function
(

	filter //RegExp:
) //iterator: returns File objects.
{
    var zipFile = Components.classes["@mozilla.org/file/local;1"]
                            .createInstance(Components.interfaces.nsILocalFile);
    var jarSpec = this._URI.spec;                        
    var i = jarSpec.indexOf("!/");
	if (i == -1) 
		throw "Not a jar file.";
		
	var zipDirSpec = jarSpec.substr(i+2);
	var zipDirSpecLength = 1;
	if (zipDirSpec != "") {
		if (!zipDirSpec.endsWith("/"))
			zipDirSpec += "/";
		zipDirSpecLength = zipDirSpec.length;
	}
	
	
	jarSpec = jarSpec.substr(0, i);

    var path = jarSpec.substr(4); // drop jar:
    path = File.URLToPath(path);	

    zipFile.initWithPath(path);

    var zipReader = Components.classes["@mozilla.org/libjar/zip-reader;1"]
                              .createInstance(Components.interfaces.nsIZipReader);
	//Notes:
	//1. If user does not consume all entries then the zipReader could have been left open.
	//2. zip iterator does NOT return names in sorted order
	//So: Read all entries first, close the reader and sort the entries.
	
    zipReader.init(zipFile);
	zipReader.open();
	zipEntries = [];
    var itr = zipReader.findEntries(zipDirSpec + "*");
    while(itr.hasMoreElements()) {
    	var entry = itr.getNext().QueryInterface(Components.interfaces.nsIZipEntry);
    	var entryName = entry.name;
		// filter only on party of entry name following Zip Dir name.	
		var entrySuffix = entryName.substr(zipDirSpecLength - 1);	
		if (filter && !filter.test(entrySuffix))
			continue;
    	zipEntries.push(entryName);
	}
	zipReader.close(); 
	itr = null; 
	zipEntries.sort();
	
	var i = 0;
	var iterator = function() {
		if (i < zipEntries.length) 
			return new File(jarSpec + "!/" + zipEntries[i++]);
		else {	
			zipEntries = [];
			return null;
		}
	}
	return iterator;
}
