/*** -*- Mode: Javascript; tab-width: 2;

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is Collabnet code.
The Initial Developer of the Original Code is Collabnet.

Portions created by Collabnet are
Copyright (C) 2000 Collabnet.  All
Rights Reserved.

Contributor(s): Pete Collins, Doug Turner, Brendan Eich, Warren Harris, Eric Plaster


 File API (The purpose of this file is to make it a little easier to do file IO from js) 

       file.js

 Function List

    open(aMode);                       // open a file handle for reading, writing or appending
    close();                           // closes a file handle
    read();                            // returns the contents of a file
    readLine(aDone);                   // returns the next line in the file.  aDone == true at EOF
    write(aContents, aPermissions);    // permissions are optional 
    eof;                               // check 'end of file' status
 Instructions:
				
	First include this following script tag in your xul file:
       	
       	<script type="application/x-javascript" src="chrome://xalib/content/XaLib.js"/>
	

		Then include this file in your js file:       	
		
		XaLib.include("chrome://xalib/content/IO/File/FileIO.js"); // Always use a chrome url.

	To create a new File object call:
    	
    	var file = new File("/path/file.ext");
    	
	    The supplied path may be a native path to your OS, a chrome: url, or a file: url.
		
	To see if the file exists, call the exists() member.  This is a good check before going into some
	deep code to try and extract information from a non-existant file. However, an exception will be thrown 
	for attempted access to non-existent files.

	To open a file for reading, writing or appending specify one of the following modes: 
		File.READ, File.WRITE or File.APPEND to  file.open(mode);

	If you want to read from a file, just call:

	  file.open(File.READ);
	  var theFilesContents    = file.read();

	The file contents will be returned to the caller so you can do something usefull with it.

	  file.close(); 

	Calling 'close()' destroys any created objects.  If you forget to use file.close() no probs
	all objects are discarded anyway.
************/
netscape.security.PrivilegeManager.enablePrivilege("UniversalXPConnect");

XaLib.include("chrome://xalib/content/IO/File/File.js");
/****************** File Object Class *********************/

// Initialize File required by this module.
try {
	// File Open modes:
	File.READ                     = 0x01;     // 1
	File.WRITE                    = 0x08;     // 8
	File.APPEND                   = 0x10;     // 16

	File.DEFAULT_PERMS            = 0644;
	
	File.CHUNK                   = 1024;     // buffer for readline => set to 1k
// Possible values for the ioFlags parameter to FileTransportService.createTransport.
// From http://lxr.mozilla.org/seamonkey/source/netwerk/base/public/nsIFileChannel.idl
	File.NS_RDONLY               = 0x01;
	File.NS_WRONLY               = 0x02;
	File.NS_RDWR                 = 0x04;
	File.NS_CREATE_fILE          = 0x08;
	File.NS_APPEND               = 0x10;
	File.NS_TRUNCATE             = 0x20;
	File.NS_SYNC                 = 0x40;
	File.NS_EXCL                 = 0x80;

	File._fileOutputStream = new Components.Constructor("@mozilla.org/network/file-output-stream;1", "nsIFileOutputStream");
	File._ScriptableInputStream = new Components.Constructor("@mozilla.org/scriptableinputstream;1", "nsIScriptableInputStream");

}
catch (e) {
	throw 'FileIO: failed to initialize.\n' + e;
}


/********************* OPEN *****************************/
File.prototype.open = function(aMode, aPerms) {
	if(this._mode)
		throw File.getMessage("open.AlreadyOpen", this._path, getStackTrace());
	this.writeCount = 0;
	this.flushCount = this.flushCount || 1; // no of writes before a flush

	aMode =	aMode || File.READ;
    aPerms = aPerms || File.DEFAULT_PERMS;
        
	var fileExists;
	if (this._localFile) {
	 	fileExists= this.exists()
	  	if(fileExists) {
	    	if(this.isDirectory()) throw File.getMessage("open.DirectoryNotAllowed", getStackTrace());
	    }
	}    

   this._mode=aMode;
	switch(this._mode) {
    case File.READ:
		try {
			// Create a file channel mode
			var channel = File._IOService.newChannelFromURI(this._URI);
			this._inputStream = new File._ScriptableInputStream();    
			this._inputStream.init(channel.open());
		} 
		catch (e) {
			this.close();
			throw File.getMessage("open.OpenInputError", this._path, e, getStackTrace());
		}
		break;
	case File.WRITE: 
   		if (!this._localFile)
   			throw File.getMessage("open.CantWriteJar", this._path, getStackTrace());
      	try {
			// Create an output stream.
			this._outputStream = new File._fileOutputStream();
      		this._outputStream.init(this._localFile, File.NS_WRONLY | File.NS_CREATE_fILE | File.NS_TRUNCATE, aPerms, 0);
	 	}
		catch(e) {
			this.close();
   			throw File.getMessage("open.OpenOutputError", this._path, e, getStackTrace());
		}	
		break;	
    case File.APPEND:
   		if (!this._localFile)
   			throw File.getMessage("open.CantAppendJar", this._path, getStackTrace());
      	try {
      		this._outputStream = new File._fileOutputStream();
      		this._outputStream.init(this._localFile, File.NS_WRONLY | File.NS_APPEND | File.NS_CREATE_fILE , aPerms, 0);
	 	}
		catch(e) {
			this.close();
			throw File.getMessage("open.OpenOutputError", this._path, e, getStackTrace());
		}
      	break;
    default:
       	throw File.getMessage("open.InvalidMode", this._mode, this._path, getStackTrace());
  }
  return this;
}

/********************* READ *****************************/
File.prototype.read = function () {
	if (!this._mode == File.READ)
		throw File.getMessage("read.ModeError", this.path, getStackTrace());
	if (!this._inputStream)
		throw File.getMessage("read.FileNotOpen", this.path, getStackTrace());

	try {
		if (this._localFile)
			return this._inputStream.read(this._localFile.fileSize);
		else {
			return this._inputStream.read(this._inputStream.available());
		}
	}
	catch (e) {
		throw File.getMessage("read.ReadError", this.path, e, getStackTrace()); 
	}
}

/********************* READLINE *****************************/
File.prototype.readLine = function() {
	if (!this._mode == File.READ)
		throw File.getMessage("read.ModeError", this.path, getStackTrace());
	if (!this._inputStream)
		throw File.getMessage("read.FileNotOpen", this.path, getStackTrace());

	try {
		if (!lineBuffer)
			lineBuffer = [];
		if(this._lineBuffer.length < 2) {
			var buf = this._inputStream.read(File.CHUNK);
			if(buf) {
				if(this._lineBuffer.length == 1) {
			  		var tmp = this._lineBuffer.shift();
			  		buf = tmp+buf;
				}
				this._lineBuffer = buf.split(/[\n\r]/);
			}
		}
		return this._lineBuffer.shift();
	}
	catch (e){ 
		throw File.getMessage("readLine.ReadError", this.path, e, getStackTrace()); 
	}
}
/********************* WRITE ****************************/
File.prototype.write = function(aBuffer) {
	if (!this._mode == File.WRITE)
		throw File.getMessage("write.ModeError", this.path, getStackTrace());
	if (!this._outputStream)
		throw File.getMessage("write.FileNotOpen", this.path, getStackTrace());
	try {
		this._outputStream.write(aBuffer, aBuffer.length);
		this.writeCount++;
		if (this.writeCount >= this.flushCount) {
			this._outputStream.flush();
			this.writeCount = 0;
		}	
	}
	catch (e) {  
		throw File.getMessage("write.WriteError", this.path, e, getStackTrace()); 
  	}

}

/********************* EOF *****************************/
File.prototype.__defineGetter__('eof', function() {
	if (!this._mode == File.READ)
		throw File.getMessage("read.ModeError", this.path, getStackTrace());
	if (!this._inputStream)
		throw File.getMessage("read.FileNotOpen", this.path, getStackTrace());

	if(this._inputStream.available() || (this._lineBuffer && this._lineBuffer.length > 0) ) 
		return false;
	else 
		return true;
  
})


/********************* CLOSE ****************************/
File.prototype.close = function() {
  /***************** Destroy Instances *********************/
	if(this._inputStream) this._inputStream = null;
	if(this._outputStream) this._outputStream = null;
	if(this._mode)         this._mode=null;
}

File.readAll = function
// Utility function to read a URI and return the contents.
(
	URI	// string: URI to read from.
	// returns string: the URI's contents.
)
{
	var theFile = new File(URI);
	theFile.open();
	var contents = theFile.read();
	theFile.close();
	return contents;
}