lexDef = {
	macros: {
			letter:/[_$\u0024\u0041-\u005a\u005f\u0061-\u007a\u00c0-\u00d6\u00d8-\u00f6\u00f8-\u00ff\u0100-\u1fff\u3040-\u318f\u3300-\u337f\u3400-\u3d2d\u4e00-\u9fff\uf900-\ufaff]/,
			digit:/[\u0030-\u0039\u0660-\u0669\u06f0-\u06f9\u0966-\u096f\u09e6-\u09ef\u0a66-\u0a6f\u0ae6-\u0aef\u0b66-\u0b6f\u0be7-\u0bef\u0c66-\u0c6f\u0ce6-\u0cef\u0d66-\u0d6f\u0e50-\u0e59\u0ed0-\u0ed9\u1040-\u1049]/,
			integer:/[0-9]+/
	},
	states: {
		normal:[
			TokenDef("ws", 			{			match:/[ \n\r\t\f]+/, variable:true, ignore:true 						} ),
			TokenDef("id", 			{			match:/<letter>(?:<letter>|<digit>)*/, variable:true 					} ),
			TokenDef("declit", 		{			match:/[+-]?<integer>\.<integer>(?:[Ee][+-]?<integer>)?/, variable:true } ),
			TokenDef("intlit",		{			match:/[+-]?<integer>/,  variable:true 									} ),
			// regexp also picks up / but when we look for a tokenDef we map to div instead of regexp
			TokenDef("regexp", 		{ 			match:/\/[^\/\\\r\n*=]*(\\.[^\/\\\r\n]*)*\/?[igm]{0,4}/gm, variable:true	} ),
			TokenDef("blkcomment", 	{id:"/*", 	match:/\/\*/, 		thenMatch:/.*?\*\//gm 								} ),
			TokenDef("sngcomment", 	{id:"//", 	match:/\/\//, 		thenMatch:/.*?$/gm 									} ),
			TokenDef("qstring", 	{id:'"', 	match:/\"/, 		thenMatch:/[^"\\\r\n]*(\\.[^"\\\r\n]*)*"/gm 		} ),
			TokenDef("astring", 	{id:"'", 	match:/\'/,         thenMatch:/[^'\\\r\n]*(\\.[^'\\\r\n]*)*'/gm 		} ),
			TokenDef("not", 		{id:"!", 	match:/!={0,2}/ 	} ),
			TokenDef("neq", 		{id:"!=", 	refid:"not" 		} ),
			TokenDef("neqiv", 		{id:"!==", 	refid:"not" 		} ),
			TokenDef("mod", 		{id:"%", 	match:/%=?/ 		} ),
			TokenDef("modeq", 		{id:"%=", 	refid:"mod" 		} ),
			TokenDef("band", 		{id:"&", 	match:/&[&=]?/		} ),
			TokenDef("land", 		{id:"&&", 	refid:"band" 		} ),
			TokenDef("bandeq", 		{id:"&=", 	refid:"band" 		} ),
			TokenDef("lparen", 		{id:"(", 	match:/\(/  		} ),
			TokenDef("rparen", 		{id:")", 	match:/\)/  		} ),
			TokenDef("mult", 		{id:"*", 	match:/\*=?/ 		} ),
			TokenDef("multeq", 		{id:"*=", 	refid:"mult" 		} ),
			TokenDef("plus", 		{id:"+", 	match:/\+[+=]?/		} ),
			TokenDef("inc", 		{id:"++", 	refid:"plus" 		} ),
			TokenDef("pluseq", 		{id:"+=", 	refid:"plus" 		} ),
			TokenDef("comma", 		{id:",", 	match:/,/  			} ),
			TokenDef("sub", 		{id:"-", 	match:/-[-=]?/		} ),
			TokenDef("dec", 		{id:"--", 	refid:"sub" 		} ),
			TokenDef("subeq", 		{id:"-=", 	refid:"sub" 		} ),
			TokenDef("dot", 		{id:".", 	match:/\.{1,3}/		} ),
			TokenDef("dot2", 		{id:"..", 	refid:"dot" 		} ),
			TokenDef("dot3", 		{id:"...", 	refid:"dot" 		} ),
			TokenDef("div", 		{id:"/", 	match:/\/(?![\/=*])/} ),
			TokenDef("diveq", 		{id:"/=", 	match:/\/=/			} ),
			TokenDef("colon", 		{id:":", 	match:/:{1,2}/		} ),
			TokenDef("colon2", 		{id:"::", 	refid:"colon" 		} ),
			TokenDef("semi", 		{id:";", 	match:/;/  			} ),
			TokenDef("lt", 			{id:"<", 	match:/[<]{1,2}=?/  } ),
			TokenDef("lshift", 		{id:"<<", 	refid:"lt" 			} ),
			TokenDef("lshifteq",	{id:"<<=", 	refid:"lt" 			} ),
			TokenDef("lteq", 		{id:"<=", 	refid:"lt" 			} ),
			TokenDef("assign", 		{id:"=", 	match:/={1,3}/		} ),
			TokenDef("eq", 			{id:"==", 	refid:"assign"		} ),
			TokenDef("equiv", 		{id:"===",	refid:"assign"		} ),
			TokenDef("gt", 			{id:">", 	match:/>{1,3}=?/	} ),
			TokenDef("gteq", 		{id:">=", 	refid:"gt"  		} ),
			TokenDef("rshift", 		{id:">>", 	refid:"gt"  		} ),
			TokenDef("rshifteq",	{id:">>=",	refid:"gt"  		} ),
			TokenDef("zrshift", 	{id:">>>",	refid:"gt"  		} ),
			TokenDef("zrshifteq",	{id:">>>=",	refid:"gt"			} ),
			TokenDef("quest", 		{id:"?", 	match:/\?/			} ),
			TokenDef("lbrkt", 		{id:"[", 	match:/\[/			} ),
			TokenDef("rbrkt", 		{id:"]", 	match:/\]/			} ),
			TokenDef("bxor", 		{id:"^", 	match:/\^/			} ),
			TokenDef("bxoreq", 		{id:"^=",	match:/\^=/			} ),
			TokenDef("lbrace", 		{id:"{", 	match:/\{/			} ),
			TokenDef("bor", 		{id:"|", 	match:/[|][|=]?/	} ),
			TokenDef("or", 			{id:"||", 	refid:"bor"  		} ),
			TokenDef("boreq", 		{id:"|=", 	refid:"bor"  		} ),
			TokenDef("rbrace", 		{id:"}", 	match:/\}/			} ),
			TokenDef("bnot", 		{id:"^", 	match:/\~/ 			} ), 
			// Keywords all use name as "id"
			TokenDef("break"),
			TokenDef("case"),
			TokenDef("catch"),
			TokenDef("class"),
			TokenDef("const"),
			TokenDef("continue"),
			TokenDef("default"),
			TokenDef("delete"),
			TokenDef("else"),
			TokenDef("eval"),
			TokenDef("extends"),
			TokenDef("field"),
			TokenDef("final"),
			TokenDef("finally"),
			TokenDef("for"),
			TokenDef("function"),
			TokenDef("if"),
			TokenDef("import"),
			TokenDef("in"),
			TokenDef("instanceof"),
			TokenDef("new"),
			TokenDef("package"),
			TokenDef("private"),
			TokenDef("protected"),
			TokenDef("public"),
			TokenDef("return"),
			TokenDef("super"),
			TokenDef("switch"),
			TokenDef("this"),
			TokenDef("throw"),
			TokenDef("try"),
			TokenDef("typeof"),
			TokenDef("var"),
			TokenDef("while"),
			TokenDef("with"),
			// literals
			TokenDef("null"),
			TokenDef("true"),
			TokenDef("false")
		]
	}
}; // end definition