XaLib.include("chrome://xalib/content/RDF/RDF.js");
//XaLib.include("chrome://xalib/content/RDF/SPARQL/Query.js");

Query.prototype.compile = function
// Prepare the query for execution.
(

)

{
	var spec = this._spec;

	// If a Prefix was specified on the query build a namespace resolver.
	if (spec.Prefix) {
		this.NSResolver = new RDF.NSResolver(spec.StartDelimiter, spec.EndDelimiter);
		for (var prefix in spec.Prefix) {
			this.NSResolver.AddNamespace(prefix, spec.Prefix[prefix]);
		}
	}
	else
		this.NSResolver = undefined;

	if (spec.From)
		this.setFromDS(spec.From);
	if (spec.With) 	
		this.setWithDS(spec.With);
		
	if (spec.Limit !=undefined) {
		this._Limit = parseInt(spec.Limit);
		if (isNaN(this._Limit) || this._Limit < 0)
			throw Query.getMessage("Compile.Limit", spec.Limit);					
	}
	else 
		this._Limit = 999999999; // Should be enough.
		
	if (spec.Distinct) {
		Number.prototype.hash = function() { return this; };
		Date.prototype.hash = function() { return this.valueOf(); };
	}

	this._varMap = {};
	this.initVars(spec.Select);
	this._tempVarCount = 0;
	// Replace multiple top level elements by single Group.
	this._topGoal;
	if (spec.Where.length == 1)
			this._topGoal = spec.Where[0];
	else {		
		this._topGoal = Group(spec.Where);
		spec.Where = [];
		spec.Where[0] = this._topGoal;
	}
	this._topGoal = this._topGoal.compileGoal(this);
}

Query.prototype.setFromDS = function
(

	fromArray
)
{
	//TODO allow nsIDataSource or string URL
	if (fromArray.length > 1) {
		DS = Components.classes["@mozilla.org/rdf/datasource;1?name=composite-datasource"]
        .createInstance(Components.interfaces.nsIRDFCompositeDataSource);
		for (var i=0; i<fromArray.length; i++) {
			var fromValue = fromArray[i];
			var fromDS;
			if (typeof(fromValue) == "string")
				fromDS = RDF.getDataSource(fromValue);
			else if (fromValue instanceof Components.interfaces.nsIRDFLiteral)	
				fromDS = RDF.getDataSource(fromValue.Value);
			else if (withValue instanceof Components.interfaces.nsIRDFDataStore)
				fromDS = fromValue;
			else
				throw Query.getMessage("Compile.From", (i+1), fromeValue);								
			DS.addDataSource(fromDS);	
		}
	}
	else {
		var fromValue = fromArray[0];
		if (typeof(fromValue) == "string")
			fromDS = RDF.getDataSource(fromValue);
		else if (fromValue instanceof Components.interfaces.nsIRDFLiteral)	
			fromDS = RDF.getDataSource(fromValue.Value);
		else if (withValue instanceof Components.interfaces.nsIRDFDataStore)
			fromDS = fromValue;
		else	
			throw Query.getMessage("Compile.From", 1, fromeValue);								
		this._dataStore = fromDS;
	}	
}
Query.prototype.setWithDS = function
(
	withArray
)
{
	for (var i=0; i<withArray.length; i++) {
		var withValue = withArray[i];
		if (typeof(withValue) == "string")
			withArray[i] = RDF.getDataSource(withValue);
		else if (withValue instanceof Components.interfaces.nsIRDFLiteral)	
			withArray[i] = RDF.getDataSource(withValue.Value);
		else if (!(withValue instanceof Components.interfaces.nsIRDFDataStore))
			throw Query.getMessage("Compile.With", (i+1), withValue);								
	}
}

Query.prototype.addVarIf = function(aVar) {
	if (aVar instanceof Var) {
		if (this._varMap[aVar.name] == undefined) {
			// It is possible that a Var has been preset to a constant.
			// 'output' means that a value is required to be produced.
			aVar.isOutput = (aVar.value == undefined); 
			if (!aVar.isExplicitValue)
				aVar.value = undefined;
			else
				aVar.value = RDF.node(aVar.value, this.NSResolver);
			this._varMap[aVar.name] = aVar;
		}
		return true;
	}
	else
		return false;
}


Query.prototype.initVars = function
(
	varArray // from the Select clause.
)
{
	//build a map of all variables in the Select clause.
	//Note: Additional Vars may be added by other goals.
	for (var i=0; i<varArray.length; i++) {
		var aVar = varArray[i];
		this.addVarIf(aVar);
	}
}
Query.prototype.newTempVar = function () {
	this._tempVarCount++;
	var aVar = new Var("_X" + this._tempVarCount);
	this.addVarIf(aVar);
	return aVar;
}
Query.prototype.compileTail = function(item) {
	var goalList;
	var lastGoal;
	var thisItem = item.nextItem;
	while (thisItem) {
		var subList = thisItem.compileGoal(this);
		if (!goalList) {
			goalList = subList;
			lastGoal = subList;
		}
		else
			lastGoal.nextGoal = subList;
		while (lastGoal.nextGoal)
			lastGoal = lastGoal.nextGoal;
		if (thisItem.isBranching)
			return goalList;
			
		thisItem = thisItem.nextItem;	
	}
	// then do parent next item.
	if (item.parent) {
		subList = this.compileTail(item.parent);
		if (lastGoal)
			lastGoal.nextGoal = subList;
		else
			goalList = subList;
	}
	
	return goalList;		
}

Group.prototype.compileGoal = function(query) {
	var childItem = this.childItem;
	var goalList =  childItem.compileGoal(query);
	if (!childItem.isBranching) {
		var lastGoal = goalList;
		while (lastGoal.nextGoal)
			lastGoal = lastGoal.nextGoal;
		lastGoal.nextGoal = query.compileTail(childItem);
	}
	return goalList;
}

Optional.prototype.compileGoal = function(query) {
	// Make goal early so that EndMarker can reference it.
	var thisGoal = new QueryEngine.Optional(null, null);
	this.thisGoal = thisGoal;
	// create map of all input vars to union.
	var childItem = this.childItem;
	var inputs = query.identifyInputs();
	var goalList =  childItem.compileGoal(query);
	if (!childItem.isBranching) {
		var lastGoal = goalList;
		while (lastGoal.nextGoal)
			lastGoal = lastGoal.nextGoal;
		lastGoal.nextGoal = query.compileTail(childItem);
	}
	// restore all output vars except those known to be inputs on entry.
	query.restoreOutputs(inputs);
	thisGoal.nextGoal = goalList;
	
	// Recompile without items in current Optional.
	thisGoal.nextAlt = new QueryEngine.Once();
	thisGoal.nextAlt.nextGoal = query.compileTail(this);
	return thisGoal;
}
EndMarker.prototype.compileGoal = function(query) {
	
	return new QueryEngine.EndMarker(this.refItem.thisGoal);
}

Union.prototype.compileGoal = function(query) {
	// create map of all input vars to union.
	var childItem = this.childItem;
	var inputs = query.identifyInputs();
	var goalList =  childItem.compileGoal(query);
	if (!childItem.isBranching) {
		var lastGoal = goalList;
		while (lastGoal.nextGoal)
			lastGoal = lastGoal.nextGoal;
		lastGoal.nextGoal = query.compileTail(childItem);
	}
	// restore all output vars except those known to be inputs on entry.
	query.restoreOutputs(inputs);

	var altGoalList;
	if  (this.nextAlt)
		altGoalList = this.nextAlt.compileGoal(query);
	return new QueryEngine.Union(goalList, altGoalList);
}

Query.prototype.identifyInputs = function() {
	var inputs = {};
	for (var varName in this._varMap) {
		var aVar = this._varMap[varName];
		if (!aVar.isOutput)
			inputs[varName] = aVar;
	}
	return inputs;
}
Query.prototype.restoreOutputs = function(inputs) {
	for (var varName in this._varMap) {
		var aVar = this._varMap[varName];
		if (inputs[varName] != undefined)
			aVar.isOutput = false; // Should not have changed.
		else
			aVar.isOutput = true; // restore to unbound.
	}
}

Graph.prototype.compileGoal = function(query) {
	var isStar = (this.graph == "*");
	if (isStar) 
		this.graph = query.newTempVar();	

	var isGVar = query.addVarIf(this.graph);
	var isGOutput = isGVar && this.graph.isOutput;

	if (isGVar) {
		if (this.graph.isOutput) {
			if (!query._spec.With || query._spec.With.length == 0)
				throw Query.getMessage("Compile.GraphNoWith");											
		}
		this.graph.isOutput = false;
	}		
	else {	
		this.graph = RDF.node(this.subject, query.NSResolver);
	}
	

	var childItem = this.childItem;
	var goalList =  childItem.compileGoal(query);
	if (!childItem.isBranching) {
		var lastGoal = goalList;
		while (lastGoal.nextGoal)
			lastGoal = lastGoal.nextGoal;
		lastGoal.nextGoal = query.compileTail(childItem);
	}
	return new QueryEngine.Graph(this.graph, goalList); 
}


Resources.prototype.compileGoal = function (query) {
	if (query.addVarIf(this.subject))
		this.subject.isOutput = false;
	else
		this.subject = RDF.node(this.subject, query.NSResolver);
	return new QueryEngine.Resources(this.subject);
}

ArcsIn.prototype.compileGoal = function (query) {
	if (query.addVarIf(this.object))
		this.object.isOutput = false;
	else
		this.object = RDF.node(this.object, query.NSResolver);

	if (query.addVarIf(this.predicate))
		this.predicate.isOutput = false;
	else
		this.predicate = RDF.node(this.predicate, query.NSResolver);
	return new QueryEngine.ArcsIn(this.object, this.predicate);
}

ArcsOut.prototype.compileGoal = function (query) {
	if (query.addVarIf(this.subject)) 
		this.subject.isOutput = false;
	else
		this.subject = RDF.node(this.subject, query.NSResolver);
		
	if (query.addVarIf(this.predicate)) 
		this.predicate.isOutput = false;
	else
		this.predicate = RDF.node(this.predicate, query.NSResolver);
	return  new QueryEngine.ArcsOut(this.subject, this.predicate);
}

Triple.prototype.compileGoal = function(query) {
	var isSVar = query.addVarIf(this.subject);
	var isPVar = query.addVarIf(this.predicate);	
	var isOVar = query.addVarIf(this.object);

	if (!isSVar) this.subject = RDF.node(this.subject, query.NSResolver);
	if (!isPVar) this.predicate = RDF.node(this.predicate, query.NSResolver);
	if (!isOVar) this.object = RDF.node(this.object, query.NSResolver);

	var isSOutput = isSVar && this.subject.isOutput;
	var isPOutput = isPVar && this.predicate.isOutput;
	var isOOutput = isOVar && this.object.isOutput;
	if (isSOutput) this.subject.isOutput = false;
	if (isPOutput) this.predicate.isOutput = false;
	if (isOOutput) this.object.isOutput = false;

	// We can optimize run time by directing to known patterns
	// We can do even better by analysing in out usage and using the appropriate pattern.
	// i.e In: (S P ?O) (?S P ?O) the second tupple is really _PO, not _P_.
	

	// Complicated by OPTION groups. 
	var goal;
	if (isSOutput && !isPOutput && !isOOutput) {
		return new QueryEngine.Triple_$PO(this.subject, this.predicate, this.object);
	}
	if (!isSOutput && !isPOutput && isOOutput) {
		return new QueryEngine.Triple_SP$(this.subject, this.predicate, this.object);
	}
	if (!isSOutput && !isPOutput && !isOOutput) {
		return  new QueryEngine.Triple_SPO(this.subject, this.predicate, this.object);
	}
	// Now it gets tricky.
	if (!isSOutput && isPOutput	&& isOOutput) {
		goal = new QueryEngine.ArcsOut(this.subject, this.predicate);
		goal.nextGoal = new QueryEngine.Triple_SP$(this.subject, this.predicate, this.object);
		return goal;
	}
	if (isSOutput && isPOutput	&& !isOOutput) {
		goal = new QueryEngine.ArcsIn(this.object, this.predicate);
		goal.nextGoal = new QueryEngine.Triple_$PO(this.subject, this.predicate, this.object);
		return goal;
	}
	if (isSOutput && !isPOutput && isOOutput) {
		goal = new QueryEngine.Resources(this.subject);
		goal.nextGoal = new QueryEngine.Triple_SP$(this.subject, this.predicate, this.object);
		return goal;
	}
	// And now the big one.
	goal = new QueryEngine.Resources(this.subject);
	goal.nextGoal = new QueryEngine.ArcsOut(this.subject, this.predicate);
	goal.nextGoal.nextGoal = new QueryEngine.Triple_SP$(this.subject, this.predicate, this.object);
	return goal;
}

In.prototype.compileGoal = function (query) {
	this.retVar.isOutput = false;
	for (var i=0; i<this.entries.length; i++) {
		if (!(this.entries[i] instanceof Var)) {
			this.entries[i] = RDF.node(this.entries[i], query.NSResolver);
		}
	}
	return new QueryEngine.In(this.retVar, this.entries);
}

Builtin.prototype.compileGoal = function(query) {
	var opr = this.args[1];
	for (var i=0; i<this.args.length; i++) {
		var arg = this.args[i];
		if (i == 1) // Don't RDF encode the operator
			continue;
		// Special case the RegExp in test and match is not an RDFNode. 
		if (i==2 && (opr == "=~" || opr == "!~" || opr == "match")) {
			if (!(arg instanceof  RegExp))
				throw Query.getMessage("Compile.Builtin1", this.name, opr, arg);											
			else continue;	
		}
		
		if (!query.addVarIf(arg))
			this.args[i] = RDF.node(arg, query.NSResolver);
	}
	// Special case processing.
	if (this.name == "Fun") {
		this.retVar = this.args[3];
		if (!(this.retVar instanceof Var)) 
			throw Query.getMessage("Compile.Builtin2", opr, this.retVar);											
		if (!this.retVar.isOutput)
			throw Query.getMessage("Compile.Builtin3", opr, this.retVar.nam);											
		this.retVar.isOutput = false;
	}
	return new QueryEngine.Builtin(this.name, this.retVar, this.args);
}

